﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;

namespace db.Utility
{
    public enum EFitImage
    {
        Crop,
        Fit
    }

    public class ImageUtility
    {
        private Size _originalSize = new Size();

        public Size OriginalSize
        {
            get { return _originalSize; }
        }

        private Size GetPreCropSize(Image img, Size boundingBox)
        {
            double whFactor = ((double)boundingBox.Width) / ((double)boundingBox.Height);
            double imgWHFactor = ((double)img.Size.Width) / ((double)img.Size.Height);
            return ((imgWHFactor >= whFactor) ? new Size((int)Math.Round(boundingBox.Height * imgWHFactor), boundingBox.Height) : new Size(boundingBox.Width, (int)Math.Round(boundingBox.Width / imgWHFactor)));
        }

        private Size GetFitSize(Image img, Size boundingBox)
        {
            double whFactor = ((double)boundingBox.Width) / ((double)boundingBox.Height);
            double imgWHFactor = ((double)img.Size.Width) / ((double)img.Size.Height);
            return ((imgWHFactor >= whFactor) ? new Size(boundingBox.Width, (int)Math.Round(boundingBox.Width / imgWHFactor)) : new Size((int)Math.Round(boundingBox.Height * imgWHFactor), boundingBox.Height));
        }

        private Size GetSize(Image img, Size boundingBox, EFitImage fitImage)
        {
            return ((fitImage == EFitImage.Crop) ? GetPreCropSize(img, boundingBox) : GetFitSize(img, boundingBox));
        }

        private Bitmap GetResizedBitmap(Image img, Size size)
        {
            Bitmap finalImg = new Bitmap(size.Width, size.Height);
            Graphics gfx = Graphics.FromImage(finalImg);
            if (img.Size.Width != size.Width || img.Size.Height != size.Height)
                gfx.InterpolationMode = InterpolationMode.HighQualityBicubic;
            gfx.DrawImage(img, new Rectangle(0, 0, size.Width, size.Height));
            gfx.Dispose();
            return finalImg;
        }

        private Bitmap GetFittedBitmap(Image img, Size size, Brush backgroundColor)
        {
            int dx = ((img.Size.Width < size.Width) ? (size.Width - img.Size.Width) / 2 : 0);
            int dy = ((img.Size.Height < size.Height) ? (size.Height - img.Size.Height) / 2 : 0);

            Bitmap finalImg = new Bitmap(size.Width, size.Height);
            Graphics gfx = Graphics.FromImage(finalImg);
            gfx.FillRectangle(backgroundColor, 0, 0, size.Width, size.Height);
            gfx.DrawImage(img, new Rectangle(dx, dy, img.Size.Width, img.Size.Height));
            gfx.Dispose();
            return finalImg;
        }

        private Bitmap GetCroppedBitmap(Image img, Size size)
        {
            int dx = ((img.Size.Width > size.Width) ? (img.Size.Width - size.Width) / 2 : 0);
            int dy = ((img.Size.Height > size.Height) ? (img.Size.Height - size.Height) / 2 : 0);

            Bitmap bmpImage = new Bitmap(img);
            Bitmap bmpCrop = bmpImage.Clone(new Rectangle(dx, dy, size.Width, size.Height), bmpImage.PixelFormat);
            bmpImage.Dispose();
            return bmpCrop;
        }

        private Bitmap GetErrorBitmap(Size size, Brush backgroundColor)
        {
            Bitmap rc = null;
            try
            {
                rc = new Bitmap(size.Width, size.Height);
                Graphics gfx = Graphics.FromImage(rc);
                gfx.FillRectangle(backgroundColor, 0, 0, size.Width, size.Height);
                gfx.DrawLine(Pens.Black, new Point(0, 0), new Point(size.Width, size.Height));
                gfx.DrawLine(Pens.Black, new Point(size.Width, 0), new Point(0, size.Height));
                gfx.Dispose();
            }
            catch (Exception) { }
            return rc;
        }

        public Bitmap GetImage(Image srcImage, Size boundingBox, EFitImage fitImage, Brush backgroundColor)
        {
            Bitmap rc = null;
            try
            {
                _originalSize = srcImage.Size;
                if (srcImage.Size.Width <= boundingBox.Width && srcImage.Size.Height <= boundingBox.Height)
                {
                    rc = GetFittedBitmap(srcImage, boundingBox, backgroundColor);
                }
                else
                {
                    if (fitImage == EFitImage.Crop)
                    {
                        Size tmpSize = GetPreCropSize(srcImage, boundingBox);
                        Bitmap tmpBmp = GetResizedBitmap(srcImage, tmpSize);
                        if (tmpBmp != null)
                        {
                            Bitmap cropped = GetCroppedBitmap(tmpBmp, boundingBox);
                            if (cropped != null)
                                rc = cropped;
                            tmpBmp.Dispose();
                        }
                    }
                    else if (fitImage == EFitImage.Fit)
                    {
                        Size tmpSize = GetFitSize(srcImage, boundingBox);
                        Bitmap resizedImage = GetResizedBitmap(srcImage, tmpSize);
                        rc = GetFittedBitmap(resizedImage, boundingBox, backgroundColor);
                        resizedImage.Dispose();
                    }
                }

                srcImage.Dispose();
            }
            catch (Exception)
            {
                rc = GetErrorBitmap(boundingBox, Brushes.Red);
            }
            return rc;
        }

        public Bitmap GetImage(Stream stream, Size boundingBox, EFitImage fitImage, Brush backgroundColor)
        {
            Bitmap rc = null;
            try
            {
                Image srcImage = Bitmap.FromStream(stream);
                rc = GetImage(srcImage, boundingBox, fitImage, backgroundColor);
                srcImage.Dispose();
            }
            catch (Exception)
            {
                rc = GetErrorBitmap(boundingBox, Brushes.Red);
            }
            return rc;
        }

        public Bitmap GetImage(byte[] byteArray, Size boundingBox, EFitImage fitImage, Brush backgroundColor)
        {
            Bitmap rc = null;
            try
            {
                Stream stream = new MemoryStream(byteArray);
                Image srcImage = Bitmap.FromStream(stream);
                rc = GetImage(srcImage, boundingBox, fitImage, backgroundColor);
                stream.Dispose();
                srcImage.Dispose();
            }
            catch (Exception)
            {
                rc = GetErrorBitmap(boundingBox, Brushes.Red);
            }
            return rc;
        }

        public Bitmap GetImage(string filePath, Size boundingBox, EFitImage fitImage, Brush backgroundColor)
        {
            Bitmap rc = null;
            try
            {
                Image srcImage = Bitmap.FromFile(filePath);
                rc = GetImage(srcImage, boundingBox, fitImage, backgroundColor);
                srcImage.Dispose();
            }
            catch (Exception)
            {
                rc = GetErrorBitmap(boundingBox, Brushes.Red);
            }
            return rc;
        }

        public static byte[] GetBytes(Bitmap bmp)
        {
            MemoryStream ms = new MemoryStream();
            bmp.Save(ms, ImageFormat.Png);
            return ms.ToArray();
        }
    }
}
